//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * The following disclaimer summarizes all the specific disclaimers *
// * of contributors to this software. The specific disclaimers,which *
// * govern, are listed with their locations in:                      *
// *   http://cern.ch/geant4/license                                  *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * This  code  implementation is the  intellectual property  of the *
// * GEANT4 collaboration.                                            *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//
// $Id: G4Muonium.cc,v 1.8 2003/06/16 16:57:55 gunter Exp $
// GEANT4 tag $Name: geant4-07-00-cand-01 $
//
// 
// ----------------------------------------------------------------------
//      GEANT 4 class implementation file
//
//      History: first implementation, based on object model of
//      4th April 1996, G.Cosmo
// **********************************************************************
//  Added particle definitions, H.Kurashige, 19 April 1996
//  Added SetCuts implementation, L.Urban, 10 May 1996
//  Operators (+=, *=, ++, -> etc.) correctly used, P. Urban, 26/6/96
//  Add MuonPlusDefinition(), H.Kurashige 4 July 1996
//  Add MuoniumDefinition(),T.K.Paraiso 7 April 2005
// ----------------------------------------------------------------------

#include <fstream>
#include <iomanip>

#include "G4Muonium.hh"

#include "G4MuonDecayChannel.hh"
#include "G4DecayTable.hh"
#include "Randomize.hh"

// ######################################################################
// ###                          MUONPLUS                              ###
// ######################################################################

G4Muonium::G4Muonium(
       const G4String&     aName,        G4double            mass,
       G4double            width,        G4double            charge,   
       G4int               iSpin,        G4int               iParity,    
       G4int               iConjugation, G4int               iIsospin,   
       G4int               iIsospin3,    G4int               gParity,
       const G4String&     pType,        G4int               lepton,      
       G4int               baryon,       G4int               encoding,
       G4bool              stable,       G4double            lifetime,
       G4DecayTable        *decaytable )
 : G4ParticleDefinition( aName,mass,width,charge,iSpin,iParity,
              iConjugation,iIsospin,iIsospin3,gParity,pType,
              lepton,baryon,encoding,stable,lifetime,decaytable )
{
  SetParticleSubType("mu");
  SetPDGStable(false);
  SetMuoniumGamma();

  //create Decay Table 
  G4DecayTable*   table = GetDecayTable();
  if (table!=NULL) delete table;
  table = new G4DecayTable();

  SetDecayTable(table);


}

// ......................................................................
// ...                 static member definitions                      ...
// ......................................................................
//     
//    Arguments for constructor are as follows
//               name             mass          width         charge
//             2*spin           parity  C-conjugation
//          2*Isospin       2*Isospin3       G-parity
//               type    lepton number  baryon number   PDG encoding
//             stable         lifetime    decay table 
G4Muonium G4Muonium::theMuonium(
		 "Mu",  0.1056584*GeV, 2.99591e-16*MeV,   0.*eplus, 
		    1,               0,             0,          
		    0,               0,             0,             
	     "lepton",              -1,             0,         -1313,
		 true,      2197.03*ns,          NULL
);

G4Muonium*  G4Muonium::MuoniumDefinition() {return &theMuonium;}

G4Muonium* G4Muonium::Muonium()
{ 
  return &theMuonium; 
}


void G4Muonium::SetMuoniumGamma()
{
  G4double gamma;
  gamma = 0.5*((1.*eplus)/(0.1056584*GeV/(c_light*c_light))-(1.*eplus)/(0.51099906*MeV/(c_light*c_light)));
  
  //  SetGyromagneticRatio(gamma);
}





