//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * The following disclaimer summarizes all the specific disclaimers *
// * of contributors to this software. The specific disclaimers,which *
// * govern, are listed with their locations in:                      *
// *   http://cern.ch/geant4/license                                  *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * This  code  implementation is the  intellectual property  of the *
// * GEANT4 collaboration.                                            *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//
// $Id: G4Muonium.hh,v 1.7 2001/10/16 08:16:15 kurasige Exp $
// GEANT4 tag $Name: geant4-07-00-cand-01 $
//
// 
// ------------------------------------------------------------
//      GEANT 4 class header file
//
//      History: first implementation, based on object model of
//      4-th April 1996, G.Cosmo
// ****************************************************************
//  Added particle definitions, H.Kurashige, 19 April 1996
//  Added SetCuts, L.Urban, 12 June 1996
//  Added not static GetEnergyCuts() and GetLengthCuts(), G.Cosmo, 11 July 1996
//  Muonium defined as a Mu, T.K.Paraiso 07-04-05
// ----------------------------------------------------------------

// Each class inheriting from G4VLepton
// corresponds to a particle type; one and only one
// instance for each class is guaranteed.

#ifndef G4Muonium_h
#define G4Muonium_h 1

#include "globals.hh"
#include "G4ios.hh"
#include "G4ParticleDefinition.hh"

// ######################################################################
// ###                         MUONPLUS                               ###
// ######################################################################

class G4Muonium : public G4ParticleDefinition
{
private: // constructors are hide as private  
  G4Muonium(
	    const G4String&     aName,        G4double            mass,
	    G4double            width,        G4double            charge,    
	    G4int               iSpin,        G4int               iParity,     
	    G4int              iConjugation, G4int               iIsospin,   
	    G4int               iIsospin3,    G4int               gParity,
	    const G4String&     pType,        G4int               lepton,      
	    G4int               baryon,       G4int               encoding,
	    G4bool              stable,       G4double            lifetime,
	    G4DecayTable        *decaytable
	    );

public:
  virtual ~G4Muonium(){;};

  void SetMuoniumGamma();

  static G4Muonium* MuoniumDefinition();
  static G4Muonium* Muonium();

private:
  static G4Muonium   theMuonium;

};


#endif






